use std::path::{Path, PathBuf};

// © 2023 John Breaux
use super::*;

/// Contains the entire AST
#[derive(Clone, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
pub struct Root(Option<PathBuf>, Vec<(usize, Line)>);
// pub struct Root { pub path: PathBuf, pub lines: Vec<Line> }

impl Root {
    pub fn file(&self) -> Option<&Path> { self.0.as_deref() }
    pub(crate) fn set_file(mut self, path: PathBuf) -> Self {
        self.0 = Some(path);
        self
    }
    pub fn lines(&self) -> &[(usize, Line)] { &self.1 }
}

impl Parsable for Root {
    fn parse<'text, T>(p: &Parser, stream: &mut T) -> Result<Self, ParseError>
    where T: TokenStream<'text> {
        let mut lines = vec![];
        loop {
            let number = stream.context().line();
            match Line::parse(p, stream)? {
                Line::EndOfFile => break,
                line => lines.push((number, line)),
            }
        }
        Ok(Root(None, lines))
    }
}

impl Display for Root {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        for (num, line) in &self.1 {
            f.pad(&format!("{num:3}: {line} "))?;
        }
        Ok(())
    }
}

impl Debug for Root {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        for line in self.0.iter() {
            Debug::fmt(line, f)?;
        }
        Ok(())
    }
}
